#line 2 "fragments.tmpl"
/*
* Copyright (c) 2022 Omar Polo <op@omarpolo.com>
*
* Permission to use, copy, modify, and distribute this software for any
* purpose with or without fee is hereby granted, provided that the above
* copyright notice and this permission notice appear in all copies.
*
* THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
* WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
* ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
* WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
* ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
* OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
#line 18 "fragments.tmpl"
#include <sys/queue.h>
#include <sys/tree.h>
#line 21 "fragments.tmpl"
#include <netinet/in.h>
#include <arpa/inet.h>
#line 24 "fragments.tmpl"
#include <event.h>
#include <libgen.h>
#include <limits.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#line 31 "fragments.tmpl"
#include "galileo.h"
#include "tmpl.h"
#line 34 "fragments.tmpl"
static int tp_navigation(struct template *);
#line 39 "fragments.tmpl"
int
tp_head(struct template *tp, const char *lang, const char *title)
{
int tp_ret = 0;
#line 41 "fragments.tmpl"
struct client *clt = tp->tp_arg;
struct proxy_config *pc = clt->clt_pc;
const char *stylesheet = pc ? pc->stylesheet : "";
const char *prfx = clt->clt_script_name;
const char *path = clt->clt_path_info;
int hidenav;
#line 48 "fragments.tmpl"
hidenav = pc ? pc->flags & PROXY_NO_NAVBAR : 0;
#line 50 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "<!doctype html>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<html")) == -1) goto err;
#line 51 "fragments.tmpl"
if (*lang != '\0') {
#line 51 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, " ")) == -1)
goto err;
#line 51 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "lang=\"")) == -1) goto err;
#line 51 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, lang)) == -1)
goto err;
#line 51 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "\"")) == -1) goto err;
}
#line 51 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, ">")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<head>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<meta name=\"viewport\" content=\"initial-scale=1\" />")) == -1) goto err;
#line 55 "fragments.tmpl"
if (*stylesheet == 0) {
if ((tp_ret = tp->tp_puts(tp, "<link rel=\"stylesheet\" href=\"")) == -1) goto err;
#line 56 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, prfx)) == -1)
goto err;
#line 56 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "galileo.css\" />")) == -1) goto err;
} else {
if ((tp_ret = tp->tp_puts(tp, "<link rel=\"stylesheet\" href=\"")) == -1) goto err;
#line 58 "fragments.tmpl"
if ((tp_ret = tp_urlescape(tp, stylesheet)) == -1)
goto err;
#line 58 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "\" />")) == -1) goto err;
}
#line 61 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "<title>")) == -1) goto err;
if (title) {
if ((tp_ret = tp->tp_escape(tp, title)) == -1)
goto err;
} else if (pc) {
if ((tp_ret = tp->tp_puts(tp, "gemini://")) == -1) goto err;
#line 65 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, pc->proxy_name)) == -1)
goto err;
#line 65 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, path)) == -1)
goto err;
}
#line 67 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "</title>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</head>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<body>")) == -1) goto err;
if (path && !hidenav) {
if ((tp_ret = tp_navigation(tp)) == -1) goto err;
}
err:
return tp_ret;
}
#line 75 "fragments.tmpl"
int
tp_foot(struct template *tp)
{
int tp_ret = 0;
#line 77 "fragments.tmpl"
struct client *clt = tp->tp_arg;
struct proxy_config *pc = clt->clt_pc;
const char *path = clt->clt_path_info;
int hidefoot;
#line 82 "fragments.tmpl"
hidefoot = pc ? pc->flags & PROXY_NO_FOOTER : 0;
#line 84 "fragments.tmpl"
if (pc && !hidefoot) {
if ((tp_ret = tp->tp_puts(tp, "<footer>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<hr />")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<dl>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<dt>Original URL:</dt>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<dd>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<a href=\"gemini://")) == -1) goto err;
#line 90 "fragments.tmpl"
if ((tp_ret = tp_urlescape(tp, pc->proxy_name)) == -1)
goto err;
#line 90 "fragments.tmpl"
if ((tp_ret = tp_urlescape(tp, path)) == -1)
goto err;
#line 90 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "\">")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "gemini://")) == -1) goto err;
#line 91 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, pc->proxy_name)) == -1)
goto err;
#line 91 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, path)) == -1)
goto err;
if ((tp_ret = tp->tp_puts(tp, "</a>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</dd>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</dl>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</footer>")) == -1) goto err;
}
#line 97 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "</body>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</html>")) == -1) goto err;
err:
return tp_ret;
}
#line 101 "fragments.tmpl"
int
tp_navigation(struct template *tp)
{
int tp_ret = 0;
#line 103 "fragments.tmpl"
struct client *clt = tp->tp_arg;
const char *dname, *dsufx = "/", *prfx = clt->clt_script_name;
char *path = clt->clt_path_info;
char *tilde, *t, home[GEMINI_MAXLEN], up[GEMINI_MAXLEN];
char usr[64];
char c;
#line 110 "fragments.tmpl"
*home = '\0';
if ((tilde = strstr(path, "/~")) != NULL &&
(t = strchr(tilde + 1, '/')) != NULL) {
c = *++t;
*t = '\0';
(void) strlcpy(home, path + 1, sizeof(home));
*t = c;
#line 118 "fragments.tmpl"
c = *--t;
*t = '\0';
(void) strlcpy(usr, tilde + 2, sizeof(usr));
*t = c;
}
#line 124 "fragments.tmpl"
(void) strlcpy(up, path + 1, sizeof(up));
dname = dirname(up);
if (!strcmp(dname, ".")) {
dname = "";
dsufx = "";
}
#line 131 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "<header>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<nav>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<ul>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<li><a href=\"")) == -1) goto err;
#line 134 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, prfx)) == -1)
goto err;
#line 134 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "\">/ Root</a></li>")) == -1) goto err;
#line 136 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "<li><a href=\"")) == -1) goto err;
#line 136 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, prfx)) == -1)
goto err;
#line 136 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, dname)) == -1)
goto err;
#line 136 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, dsufx)) == -1)
goto err;
#line 136 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "\">")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "↑ Up")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</a></li>")) == -1) goto err;
#line 140 "fragments.tmpl"
if (*home != '\0') {
if ((tp_ret = tp->tp_puts(tp, "<li><a href=\"")) == -1) goto err;
#line 141 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, prfx)) == -1)
goto err;
#line 141 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, home)) == -1)
goto err;
#line 141 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "\">")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "🏠 ")) == -1) goto err;
#line 142 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, usr)) == -1)
goto err;
#line 142 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "' home")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</a></li>")) == -1) goto err;
}
#line 145 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "</ul>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</nav>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</header>")) == -1) goto err;
err:
return tp_ret;
}
#line 150 "fragments.tmpl"
int
tp_figure(struct template *tp, const char *url, const char *label)
{
int tp_ret = 0;
if ((tp_ret = tp->tp_puts(tp, "<figure>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<a href=\"")) == -1) goto err;
#line 152 "fragments.tmpl"
if ((tp_ret = tp_urlescape(tp, url)) == -1)
goto err;
#line 152 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "\">")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<img src=\"")) == -1) goto err;
#line 153 "fragments.tmpl"
if ((tp_ret = tp_urlescape(tp, url)) == -1)
goto err;
#line 153 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "\" />")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</a>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<figcaption>")) == -1) goto err;
if ((tp_ret = tp->tp_escape(tp, label)) == -1)
goto err;
if ((tp_ret = tp->tp_puts(tp, "</figcaption>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</figure>")) == -1) goto err;
err:
return tp_ret;
}
#line 161 "fragments.tmpl"
int
tp_pre_open(struct template *tp, const char *label)
{
int tp_ret = 0;
if ((tp_ret = tp->tp_puts(tp, "<figure>")) == -1) goto err;
if (label && *label != '\0') {
if ((tp_ret = tp->tp_puts(tp, "<figcaption>")) == -1) goto err;
#line 164 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, label)) == -1)
goto err;
#line 164 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "</figcaption>")) == -1) goto err;
}
#line 166 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "<pre>")) == -1) goto err;
err:
return tp_ret;
}
#line 169 "fragments.tmpl"
int
tp_pre_close(struct template *tp)
{
int tp_ret = 0;
if ((tp_ret = tp->tp_puts(tp, "</pre>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</figure>")) == -1) goto err;
err:
return tp_ret;
}
#line 174 "fragments.tmpl"
int
tp_error(struct template *tp, int code, const char *reason)
{
int tp_ret = 0;
#line 176 "fragments.tmpl"
char scode[32];
int r;
#line 179 "fragments.tmpl"
r = snprintf(scode, sizeof(code), "%d", code);
if (r < 0 || (size_t)r >= sizeof(code))
return (0);
#line 184 "fragments.tmpl"
if ((tp_ret = tp_head(tp, "en", "Proxy error")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<main>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<h1>Proxy error</h1>")) == -1) goto err;
if (code != -1) {
if ((tp_ret = tp->tp_puts(tp, "<p>Request failed with code: <code>")) == -1) goto err;
#line 188 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, scode)) == -1)
goto err;
#line 188 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "</code>.</p>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<p>The server says: ")) == -1) goto err;
#line 189 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, reason)) == -1)
goto err;
#line 189 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, ".</p>")) == -1) goto err;
} else {
if ((tp_ret = tp->tp_puts(tp, "<p>Unable to serve the page due to: ")) == -1) goto err;
#line 191 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, reason)) == -1)
goto err;
#line 191 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, ".</p>")) == -1) goto err;
}
#line 193 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "</main>")) == -1) goto err;
if ((tp_ret = tp_foot(tp)) == -1) goto err;
err:
return tp_ret;
}
#line 197 "fragments.tmpl"
int
tp_inputpage(struct template *tp, const char *prompt)
{
int tp_ret = 0;
if ((tp_ret = tp_head(tp, "en", "input request")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<p>The server ask for input: <q>")) == -1) goto err;
#line 199 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, prompt)) == -1)
goto err;
#line 199 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "</q>.</p>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<form method=\"post\" enctype=\"")) == -1) goto err;
#line 200 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, FORM_URLENCODED)) == -1)
goto err;
#line 200 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "\">")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<label for=\"reply\">")) == -1) goto err;
#line 201 "fragments.tmpl"
if ((tp_ret = tp->tp_escape(tp, "response ")) == -1)
goto err;
#line 201 "fragments.tmpl"
if ((tp_ret = tp->tp_puts(tp, "</label>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "<input type=\"text\" value=\"\" id=\"reply\" name=\"q\" autofocus />")) == -1) goto err;
if ((tp_ret = tp->tp_escape(tp, " ")) == -1)
goto err;
if ((tp_ret = tp->tp_puts(tp, "<button type=\"submit\">Submit!</button>")) == -1) goto err;
if ((tp_ret = tp->tp_puts(tp, "</form>")) == -1) goto err;
if ((tp_ret = tp_foot(tp)) == -1) goto err;
err:
return tp_ret;
}
